@@ -1,4 +1,4 @@
-# Copyright (c) 2008-2009 Simplistix Ltd
+# Copyright (c) 2008-2012 Simplistix Ltd
 #
 # This Software is released under the MIT License:
 # http://www.opensource.org/licenses/mit-license.html
@@ -10,7 +10,8 @@ from tempfile import TemporaryFile
 from testfixtures import compare,Comparison as C,should_raise,replace,log_capture,tempdir
 from unittest import TestSuite,TestCase,makeSuite
 from xlrd import open_workbook,XL_CELL_NUMBER,XL_CELL_ERROR,XL_CELL_TEXT,XL_CELL_BOOLEAN
-from xlutils.filter import BaseReader,GlobReader,MethodFilter,BaseWriter,process,XLRDReader,XLWTWriter
+from xlrd.formatting import XF
+from xlutils.filter import BaseReader,GlobReader,MethodFilter,BaseWriter,process,XLRDReader,XLWTWriter, BaseFilter
 from xlutils.tests.fixtures import test_files,test_xls_path,make_book,make_sheet,DummyBook
 
 import os
@@ -29,13 +30,13 @@ class TestReader(BaseReader):
         for name,value in sorted(books.items()):
             self.makeBook(name,value)
         
-    def makeBook(self,sheet_name,sheets):
+    def makeBook(self,book_name,sheets):
         book = DummyBook(self.formatting_info)
         index = 0
         for name,rows in sheets:
             make_sheet(rows,book,name,index)
             index+=1
-        self.books.append((book,sheet_name+'.xls'))
+        self.books.append((book,book_name+'.xls'))
         
     def get_workbooks(self):
         return self.books
@@ -48,6 +49,18 @@ class TestBaseReader(TestCase):
         should_raise(r,NotImplementedError())(f)
         self.assertEqual(f.method_calls,[('start',(),{})])
         
+    def test_ragged_rows(self):
+        class TestReader(BaseReader):
+            def get_filepaths(self):
+                return (os.path.join(test_files,'ragged.xls'),)
+        t = TestReader()
+        class TestFilter(BaseFilter):
+            def cell(self,rdrowx,rdcolx,wtrowx,wtcolx):
+                self.rdsheet.cell(rdrowx,rdcolx)
+        f = TestFilter()
+        f.next = Mock()
+        t(f)
+    
     def test_custom_filepaths(self):
         # also tests the __call__ method
         class TestReader(BaseReader):
@@ -62,6 +75,7 @@ class TestBaseReader(TestCase):
                            pickleable=0,
                            formatting_info=1,
                            on_demand=True,
+                           ragged_rows=True,
                            strict=False),'test.xls'),{}),
             ('sheet',(C('xlrd.sheet.Sheet'),u'Sheet1'),{}),
             ('row',(0,0),{}),
@@ -108,9 +122,11 @@ class TestBaseReader(TestCase):
             book.sheet_by_index.side_effect = self.sheet_by_index
             book.sheet0.nrows=1
             book.sheet0.ncols=1
+            book.sheet0.row_len.return_value = 1
             book.sheet0.name='sheet0'
             book.sheet1.nrows=1
             book.sheet1.ncols=1
+            book.sheet1.row_len.return_value = 1
             book.sheet1.name='sheet1'
             self.b = book
             
@@ -133,11 +149,13 @@ class TestBaseReader(TestCase):
             ('book.sheet_by_index', (0,), {}),
             ('filter.sheet',(book.sheet0, 'sheet0'),{}),
             ('filter.row', (0, 0), {}),
+            ('book.sheet0.row_len', (0,), {}),
             ('filter.cell', (0, 0, 0, 0), {}),
             ('book.unload_sheet', (0,), {}),
             ('book.sheet_by_index', (1,), {}),
             ('filter.sheet',(book.sheet1, 'sheet1'),{}),
             ('filter.row', (0, 0), {}),
+            ('book.sheet1.row_len', (0,), {}),
             ('filter.cell', (0, 0, 0, 0), {}),
             ('book.unload_sheet', (1,), {}),
             ('filter.finish', (), {})
@@ -156,10 +174,12 @@ class TestBaseReader(TestCase):
             ('book.sheet_by_index', (0,), {}),
             ('filter.sheet',(book.sheet0, 'sheet0'),{}),
             ('filter.row', (0, 0), {}),
+            ('book.sheet0.row_len', (0,), {}),
             ('filter.cell', (0, 0, 0, 0), {}),
             ('book.sheet_by_index', (1,), {}),
             ('filter.sheet',(book.sheet1, 'sheet1'),{}),
             ('filter.row', (0, 0), {}),
+            ('book.sheet1.row_len', (0,), {}),
             ('filter.cell', (0, 0, 0, 0), {}),
             ('filter.finish', (), {})
             ])
@@ -402,6 +422,7 @@ class TestErrorFilter(TestCase):
                            pickleable=0,
                            formatting_info=1,
                            on_demand=False,
+                           ragged_rows=True,
                            strict=False),'test.xls'),{}),
             ('sheet',(C('xlrd.sheet.Sheet'),u'Sheet1'),{}),
             ('row',(0,0),{}),
@@ -683,7 +704,7 @@ class TestColumnTrimmer(TestCase):
 
     @log_capture()
     def test_multiple_books(self,h):
-        r = GlobReader(os.path.join(test_files,'*.xls'))
+        r = GlobReader(os.path.join(test_files,'test*.xls'))
         book = tuple(r.get_workbooks())[0][0]
         # fire methods on filter
         f = ColumnTrimmer()
@@ -712,9 +733,8 @@ class TestColumnTrimmer(TestCase):
             ('row', (5, 5),{}),
             ('cell', (0, 0, 0, 0),{}),('cell', (0, 1, 0, 1),{}),
             ('cell', (1, 0, 1, 0),{}),('cell', (1, 1, 1, 1),{}),
-            ('cell', (2, 0, 2, 0),{}),('cell', (2, 1, 2, 1),{}),
-            ('cell', (3, 0, 3, 0),{}),('cell', (3, 1, 3, 1),{}),
-            ('cell', (4, 0, 4, 0),{}),('cell', (4, 1, 4, 1),{}),
+            ('cell', (2, 0, 2, 0),{}),
+            ('cell', (3, 0, 3, 0),{}),
             ('cell', (5, 0, 5, 0),{}),('cell', (5, 1, 5, 1),{}),
             ('sheet', (C('xlrd.sheet.Sheet'), u'Sheet2'),{}),
             ('row', (0, 0),{}),
@@ -734,13 +754,7 @@ class TestColumnTrimmer(TestCase):
             ('cell', (1, 0, 1, 0), {}),
             ('cell', (1, 1, 1, 1), {}),
             ('cell', (2, 0, 2, 0), {}),
-            ('cell', (2, 1, 2, 1), {}),
-            ('cell', (3, 0, 3, 0), {}),
-            ('cell', (3, 1, 3, 1), {}),
-            ('cell', (4, 0, 4, 0), {}),
-            ('cell', (4, 1, 4, 1), {}),
             ('cell', (5, 0, 5, 0), {}),
-            ('cell', (5, 1, 5, 1), {}),
             ('sheet', (C('xlrd.sheet.Sheet'), u'Sheet2'), {}),
             ('row', (0, 0), {}),
             ('row', (1, 1), {}),
@@ -814,8 +828,8 @@ class TestBaseWriter(TestCase):
     def check_file(self,writer,path,
                    l_a_xf_list=20,
                    l_e_xf_list=26,
-                   l_a_format_map=38,
-                   l_e_format_map=37,
+                   l_a_format_map=76,
+                   l_e_format_map=75,
                    l_a_font_list=9,
                    l_e_font_list=7,
                    **provided_overrides):
@@ -872,8 +886,8 @@ class TestBaseWriter(TestCase):
                 'first_visible_rowx',
                 'first_visible_colx',
                 'gridline_colour_index',
-                'cached_page_break_preview_mag_factor',
-                'cached_normal_view_mag_factor',
+                'cooked_page_break_preview_mag_factor',
+                'cooked_normal_view_mag_factor',
                 'default_row_height',
                 'default_row_height_mismatch',
                 'default_row_hidden',
@@ -882,6 +896,11 @@ class TestBaseWriter(TestCase):
                 'nrows',
                 'ncols',
                 'standardwidth',
+                'vert_split_pos',
+                'horz_split_pos',
+                'vert_split_first_visible',
+                'horz_split_first_visible',
+                'split_active_pane',
                 )
             for col_x in range(ash.ncols):
                 ac = ash.colinfo_map.get(col_x)
@@ -980,8 +999,8 @@ class TestBaseWriter(TestCase):
             
         # xlwt writes more formats than exist in an original,
         # but we check those copied are identical
-        self.assertEqual(len(a.format_map),l_a_format_map)
-        self.assertEqual(len(e.format_map),l_e_format_map)
+        self.assertEqual(len(a.format_map), l_a_format_map)
+        self.assertEqual(len(e.format_map), l_e_format_map)
         for ai,eis in self.noted_indexes['format_key'].items():
             af = a.format_map[ai]
             for ei in eis:
@@ -1048,19 +1067,13 @@ class TestBaseWriter(TestCase):
         self.check_file(w,test_xls_path,
                         l_a_xf_list=17,
                         l_e_xf_list=17,
-                        l_a_format_map=37,
+                        l_a_format_map=75,
                         l_a_font_list=6,
-                        l_e_font_list=6,
-                        sheet=dict(
-                            # as of xlwt 0.7.2, these default values
-                            # are written if none are provided :-(
-                            cached_page_break_preview_mag_factor=60,
-                            cached_normal_view_mag_factor=100,
-                            ))
+                        l_e_font_list=6)
 
     def test_multiple_workbooks(self):
         # globreader is tested elsewhere
-        r = GlobReader(os.path.join(test_files,'*.xls'))
+        r = GlobReader(os.path.join(test_files,'test*.xls'))
         # send straight to writer
         w = TestWriter()
         r(w)
@@ -1071,9 +1084,9 @@ class TestBaseWriter(TestCase):
         self.failUnless('testnoformatting.xls' in w.closed)
         self.check_file(w,os.path.join(test_files,'testall.xls'))
         self.check_file(w,os.path.join(test_files,'test.xls'),
-                        18,21,38,37,7,4)
+                        18,21,76,75,7,4)
         self.check_file(w,os.path.join(test_files,'testnoformatting.xls'),
-                        18,17,37,37,6,6)
+                        18,17,75,75,6,6)
     
     def test_start(self):
         w = TestWriter()
@@ -1174,6 +1187,60 @@ class TestBaseWriter(TestCase):
         a = open_workbook(file_contents=f.read(),pickleable=0,formatting_info=1)
         self.assertEqual(a.sheet_names(),[name])
         
+    def test_panes(self):
+        r = TestReader()
+        r.formatting_info = True
+        
+        r.setup(('sheet',[['S1R0C0']]))
+        
+        book = tuple(r.get_workbooks())[0][0]
+        sheet = book.sheet_by_index(0)
+        sheet.panes_are_frozen = 1
+        sheet.has_pane_record = True
+        sheet.vert_split_pos = 1
+        sheet.horz_split_pos = 2
+        sheet.vert_split_first_visible = 3
+        sheet.horz_split_first_visible = 4
+        sheet.split_active_pane = 3
+        
+        w = TestWriter()
+        r(w)
+        
+        self.assertEqual(w.files.keys(),['test.xls'])
+        f = w.files['test.xls'].file
+        a = open_workbook(file_contents=f.read(),formatting_info=1)
+        sheet = a.sheet_by_index(0)
+        self.assertEqual(1,sheet.panes_are_frozen)
+        self.assertEqual(1,sheet.has_pane_record)
+        self.assertEqual(1,sheet.vert_split_pos)
+        self.assertEqual(2,sheet.horz_split_pos)
+        self.assertEqual(3,sheet.vert_split_first_visible)
+        self.assertEqual(4,sheet.horz_split_first_visible)
+        self.assertEqual(3,sheet.split_active_pane)
+        
+    def test_zoom_factors(self):
+        r = TestReader()
+        r.formatting_info = True
+        
+        r.setup(('sheet',[['S1R0C0']]))
+        
+        book = tuple(r.get_workbooks())[0][0]
+        sheet = book.sheet_by_index(0)
+        sheet.cooked_normal_view_mag_factor = 33
+        sheet.cooked_page_break_preview_mag_factor = 44
+        sheet.show_in_page_break_preview = True
+
+        w = TestWriter()
+        r(w)
+        
+        self.assertEqual(w.files.keys(),['test.xls'])
+        f = w.files['test.xls'].file
+        a = open_workbook(file_contents=f.read(),formatting_info=1)
+        sheet = a.sheet_by_index(0)
+        self.assertEqual(33,sheet.cooked_normal_view_mag_factor)
+        self.assertEqual(44,sheet.cooked_page_break_preview_mag_factor)
+        self.assertEqual(1,sheet.show_in_page_break_preview)
+        
     def test_excessive_length_sheet_name(self):
         r = TestReader(
             ('X'*32,([['S1R0C0']]),),
@@ -1217,7 +1284,7 @@ class TestDirectoryWriter(TestCase):
         from xlutils.filter import DirectoryWriter
         self.failUnless(isinstance(DirectoryWriter('foo'),BaseWriter))
 
-    @tempdir
+    @tempdir()
     def test_plus_in_workbook_name(self,d):
         from xlutils.filter import DirectoryWriter
         r = TestReader(
@@ -1258,7 +1325,7 @@ class TestXLWTWriter(TestCase):
         compare(self.w.wtbook,None)
         
     def test_multiple_files(self):
-        r = GlobReader(os.path.join(test_files,'*.xls'))
+        r = GlobReader(os.path.join(test_files,'test*.xls'))
         r(self.w)
         compare(self.w.output,[
             ('test.xls',C('xlwt.Workbook')),
@@ -1393,12 +1460,16 @@ class TestTestReader(TestCase):
         book = r.books[0][0]
         sx,rx,cx = 0,0,0
         book.sheet_by_index(sx)._cell_xf_indexes[rx][cx]=42
-
         # NB: cells where you haven't specified an xf index manually as
         #     above will have an xf index of 0:
         compare(book.sheet_by_index(0).cell(0,1).xf_index,0)
-        # and no matching style:
-        should_raise(book.xf_list,IndexError)[0]
+
+        # NB: when formattng is turned on, an XF will be created at index zero:
+        compare(C(XF),book.xf_list[0])
+        # ...but no others:
+        should_raise(book.xf_list,IndexError)[42]
+        # so you'll need to manually create them if you need them.
+        # See fixtures.py for examples.
 
         r(f)
         
